(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms'), require('@tiptap/extension-floating-menu'), require('@tiptap/extension-bubble-menu'), require('@tiptap/core')) :
    typeof define === 'function' && define.amd ? define('ngx-tiptap', ['exports', '@angular/core', '@angular/forms', '@tiptap/extension-floating-menu', '@tiptap/extension-bubble-menu', '@tiptap/core'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ngx-tiptap'] = {}, global.ng.core, global.ng.forms, global.tiptapFloatingMenu, global.tiptapBubbleMenu, global.tiptapCore));
}(this, (function (exports, core, forms, extensionFloatingMenu, extensionBubbleMenu, core$1) { 'use strict';

    var EditorDirective = /** @class */ (function () {
        function EditorDirective(el, _renderer) {
            var _this = this;
            this.el = el;
            this._renderer = _renderer;
            this.outputFormat = 'html';
            this.onChange = function () { };
            this.onTouched = function () { };
            this.handleChange = function (_a) {
                var transaction = _a.transaction;
                if (!transaction.docChanged) {
                    return;
                }
                if (_this.outputFormat === 'html') {
                    _this.onChange(_this.editor.getHTML());
                    return;
                }
                _this.onChange(_this.editor.getJSON());
            };
        }
        // Writes a new value to the element.
        // This methods is called when programmatic changes from model to view are requested.
        EditorDirective.prototype.writeValue = function (value) {
            if (!value) {
                return;
            }
            if (!this.outputFormat && typeof value === 'string') {
                this.outputFormat = 'html';
            }
            this.editor.chain().setContent(value, false).run();
        };
        // Registers a callback function that is called when the control's value changes in the UI.
        EditorDirective.prototype.registerOnChange = function (fn) {
            this.onChange = fn;
        };
        // Registers a callback function that is called by the forms API on initialization to update the form model on blur.
        EditorDirective.prototype.registerOnTouched = function (fn) {
            this.onTouched = fn;
        };
        // Called by the forms api to enable or disable the element
        EditorDirective.prototype.setDisabledState = function (isDisabled) {
            this.editor.setEditable(!isDisabled);
            this._renderer.setProperty(this.el.nativeElement, 'disabled', isDisabled);
        };
        EditorDirective.prototype.ngOnInit = function () {
            var _this = this;
            if (!this.editor) {
                throw new Error('Required: Input `editor`');
            }
            // take the inner contents and clear the block
            var innerHTML = this.el.nativeElement.innerHTML;
            this.el.nativeElement.innerHTML = '';
            // insert the editor in the dom
            this.el.nativeElement.appendChild(this.editor.options.element.firstChild);
            // update the options for the editor
            this.editor.setOptions({ element: this.el.nativeElement });
            // update content to the editor
            if (innerHTML) {
                this.editor.chain().setContent(innerHTML, false).run();
            }
            // register blur handler to update `touched` property
            this.editor.on('blur', function () {
                _this.onTouched();
            });
            // register transaction handler to emit changes on update
            this.editor.on('transaction', this.handleChange);
        };
        EditorDirective.prototype.ngOnDestroy = function () {
            this.editor.destroy();
        };
        return EditorDirective;
    }());
    EditorDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: 'tiptap[editor], [tiptap][editor], tiptap-editor[editor], [tiptapEditor][editor]',
                    providers: [{
                            provide: forms.NG_VALUE_ACCESSOR,
                            useExisting: core.forwardRef(function () { return EditorDirective; }),
                            multi: true
                        }]
                },] }
    ];
    EditorDirective.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: core.Renderer2 }
    ]; };
    EditorDirective.propDecorators = {
        editor: [{ type: core.Input }],
        outputFormat: [{ type: core.Input }]
    };

    var FloatingMenuDirective = /** @class */ (function () {
        function FloatingMenuDirective(_el) {
            this._el = _el;
            this.tippyOptions = {};
        }
        FloatingMenuDirective.prototype.ngOnInit = function () {
            if (!this.editor) {
                throw new Error('Required: Input `editor`');
            }
            this.editor.registerPlugin(extensionFloatingMenu.FloatingMenuPlugin({
                editor: this.editor,
                element: this._el.nativeElement,
                tippyOptions: this.tippyOptions
            }));
        };
        FloatingMenuDirective.prototype.ngOnDestroy = function () {
            this.editor.unregisterPlugin(extensionFloatingMenu.FloatingMenuPluginKey);
        };
        return FloatingMenuDirective;
    }());
    FloatingMenuDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: 'tiptap-floting-menu[editor], [tiptapFloatingMenu][editor]'
                },] }
    ];
    FloatingMenuDirective.ctorParameters = function () { return [
        { type: core.ElementRef }
    ]; };
    FloatingMenuDirective.propDecorators = {
        editor: [{ type: core.Input }],
        tippyOptions: [{ type: core.Input }]
    };

    var BubbleMenuDirective = /** @class */ (function () {
        function BubbleMenuDirective(_el) {
            this._el = _el;
            this.tippyOptions = {};
        }
        BubbleMenuDirective.prototype.ngOnInit = function () {
            if (!this.editor) {
                throw new Error('Required: Input `editor`');
            }
            this.editor.registerPlugin(extensionBubbleMenu.BubbleMenuPlugin({
                editor: this.editor,
                element: this._el.nativeElement,
                tippyOptions: this.tippyOptions
            }));
        };
        BubbleMenuDirective.prototype.ngOnDestroy = function () {
            this.editor.unregisterPlugin(extensionBubbleMenu.BubbleMenuPluginKey);
        };
        return BubbleMenuDirective;
    }());
    BubbleMenuDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: 'tiptap-bubble-menu[editor], [tiptapBubbleMenu][editor]'
                },] }
    ];
    BubbleMenuDirective.ctorParameters = function () { return [
        { type: core.ElementRef }
    ]; };
    BubbleMenuDirective.propDecorators = {
        editor: [{ type: core.Input }],
        tippyOptions: [{ type: core.Input }]
    };

    var NgxTiptapModule = /** @class */ (function () {
        function NgxTiptapModule() {
        }
        return NgxTiptapModule;
    }());
    NgxTiptapModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: [
                        EditorDirective,
                        FloatingMenuDirective,
                        BubbleMenuDirective,
                    ],
                    exports: [
                        EditorDirective,
                        FloatingMenuDirective,
                        BubbleMenuDirective
                    ]
                },] }
    ];

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        Object.defineProperty(o, k2, { enumerable: true, get: function () { return m[k]; } });
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    /** @deprecated */
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    /** @deprecated */
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    function __spreadArray(to, from) {
        for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
            to[j] = from[i];
        return to;
    }
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, state, kind, f) {
        if (kind === "a" && !f)
            throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
            throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    }
    function __classPrivateFieldSet(receiver, state, value, kind, f) {
        if (kind === "m")
            throw new TypeError("Private method is not writable");
        if (kind === "a" && !f)
            throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
            throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    }

    var AngularNodeViewComponent = /** @class */ (function () {
        function AngularNodeViewComponent() {
        }
        return AngularNodeViewComponent;
    }());
    AngularNodeViewComponent.decorators = [
        { type: core.Component, args: [{ template: '' },] }
    ];
    AngularNodeViewComponent.propDecorators = {
        props: [{ type: core.Input }]
    };
    var AngularNodeView = /** @class */ (function (_super) {
        __extends(AngularNodeView, _super);
        function AngularNodeView() {
            var _this = _super.apply(this, __spread(arguments)) || this;
            _this.updateProps = function (props) {
                _this.componentRef.instance.props = Object.assign(Object.assign({}, _this.componentRef.instance.props), props);
            };
            return _this;
        }
        AngularNodeView.prototype.mount = function () {
            var _this = this;
            var injector = this.options.injector;
            this.applicationRef = injector.get(core.ApplicationRef);
            var componentFactoryResolver = injector.get(core.ComponentFactoryResolver);
            var factory = componentFactoryResolver.resolveComponentFactory(this.component);
            this.componentRef = factory.create(injector, []);
            // Attach to the view so that the change detector knows to run
            this.applicationRef.attachView(this.componentRef.hostView);
            var props = {
                editor: this.editor,
                node: this.node,
                decorations: this.decorations,
                selected: false,
                extension: this.extension,
                getPos: function () { return _this.getPos(); },
                updateAttributes: function (attributes) {
                    if (attributes === void 0) { attributes = {}; }
                    return _this.updateAttributes(attributes);
                },
            };
            // Pass input props to the component
            this.componentRef.instance.props = props;
            this.renderer = this.componentRef.injector.get(core.ElementRef);
            this.contentDOMElement = this.node.isLeaf ? null : document.createElement(this.node.isInline ? 'span' : 'div');
            if (this.contentDOMElement) {
                // For some reason the whiteSpace prop is not inherited properly in Chrome and Safari
                // With this fix it seems to work fine
                // See: https://github.com/ueberdosis/tiptap/issues/1197
                // this.contentDOMElement.style.whiteSpace = 'inherit'
            }
        };
        Object.defineProperty(AngularNodeView.prototype, "dom", {
            get: function () {
                return this.renderer.nativeElement;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(AngularNodeView.prototype, "contentDOM", {
            get: function () {
                if (this.node.isLeaf) {
                    return null;
                }
                var contentElement = this.dom.querySelector('[data-node-view-content]');
                if (this.contentDOMElement
                    && contentElement
                    && !contentElement.contains(this.contentDOMElement)) {
                    contentElement.appendChild(this.contentDOMElement);
                }
                return this.contentDOMElement;
            },
            enumerable: false,
            configurable: true
        });
        AngularNodeView.prototype.update = function (node, decorations) {
            if (this.options.update) {
                return this.options.update(node, decorations);
            }
            if (node.type !== this.node.type) {
                return false;
            }
            if (node === this.node && this.decorations === decorations) {
                return true;
            }
            this.node = node;
            this.decorations = decorations;
            this.updateProps({ node: node, decorations: decorations });
            return true;
        };
        AngularNodeView.prototype.stopEvent = function (event) {
            var _a, _b, _c;
            return (_c = (_b = (_a = this.options).stopEvent) === null || _b === void 0 ? void 0 : _b.call(_a, event)) !== null && _c !== void 0 ? _c : null;
        };
        AngularNodeView.prototype.selectNode = function () {
            this.updateProps({ selected: true });
        };
        AngularNodeView.prototype.deselectNode = function () {
            this.updateProps({ selected: false });
        };
        AngularNodeView.prototype.destroy = function () {
            this.applicationRef.detachView(this.componentRef.hostView);
        };
        return AngularNodeView;
    }(core$1.NodeView));
    var AngularNodeViewRenderer = function (component, options) {
        return function (props) {
            return new AngularNodeView(component, props, options);
        };
    };

    /*
     * Public API Surface of ngx-tiptap
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.AngularNodeViewComponent = AngularNodeViewComponent;
    exports.AngularNodeViewRenderer = AngularNodeViewRenderer;
    exports.BubbleMenuDirective = BubbleMenuDirective;
    exports.EditorDirective = EditorDirective;
    exports.FloatingMenuDirective = FloatingMenuDirective;
    exports.NgxTiptapModule = NgxTiptapModule;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-tiptap.umd.js.map
